@@ -6,12 +6,19 @@
 
 #include <vector>
 #include <stdexcept>
-#include <boost/python.hpp>
 
 
+// https://stackoverflow.com/a/44175911
+class noncopyable {
+public:
+  noncopyable() = default;
+  ~noncopyable() = default;
 
+private:
+  noncopyable(const noncopyable&) = delete;
+  noncopyable& operator=(const noncopyable&) = delete;
+};
 
-namespace {
 
 class tSizeChangeNotifier;
 
@@ -72,7 +79,7 @@ class tSizeChangeNotifier
 
 template<class ElementT>
 class tReadOnlyForeignArray : public tSizeChangeNotifier, public tSizeChangeNotificationReceiver,
-  public boost::noncopyable
+  public noncopyable
 {
   protected:
     ElementT                    *&Contents;
@@ -274,9 +281,6 @@ class tForeignArray : public tReadOnlyForeignArray<ElementT>
     }
 };
 
-}
-
-
 
 
 #endif
